type IsAny<T> = 0 extends T & 1 ? true : false;

type NonEmptyArray<T> = [T, ...Array<T>];
type Mapped<T extends IterableContainer, K> = {
    -readonly [P in keyof T]: K;
};
/**
 * This should only be used for defining generics which extend any kind of JS
 * array under the hood, this includes arrays *AND* tuples (of the form [x, y],
 * and of the form [x, ...y[]], etc...), and their readonly equivalent. This
 * allows us to be more inclusive to what functions can process.
 *
 * @example map<T extends ArrayLike>(items: T) { ... }
 *
 * We would've named this `ArrayLike`, but that's already used by typescript...
 * @see This was inspired by the type-definition of Promise.all (https://github.com/microsoft/TypeScript/blob/1df5717b120cddd325deab8b0f2b2c3eecaf2b01/src/lib/es2015.promise.d.ts#L21)
 */
type IterableContainer<T = unknown> = ReadonlyArray<T> | readonly [];
type ObjectKeys<T extends object> = `${Exclude<keyof T, symbol>}`;
/**
 * Copied verbatim from `sindresorhus/ts-extras` (MIT License).
 *
 * @see https://github.com/sindresorhus/type-fest/blob/main/source/readonly-tuple.d.ts
 */
type ReadonlyTuple<Element, Length extends number> = number extends Length ? ReadonlyArray<Element> : BuildTupleHelper<Element, Length, []>;
/**
 * An extension of Extract for type predicates which falls back to the base
 * in order to narrow the `unknown` case.
 *
 * @example
 *   function isMyType<T>(data: T | MyType): data is NarrowedTo<T, MyType> { ... }
 */
type NarrowedTo<T, Base> = Extract<T, Base> extends never ? Base : IsAny<T> extends true ? Base : Extract<T, Base>;
type BuildTupleHelper<Element, Length extends number, Rest extends Array<Element>> = Rest["length"] extends Length ? readonly [...Rest] : BuildTupleHelper<Element, Length, [Element, ...Rest]>;
type ExactRecord<Key extends PropertyKey, Value> = string extends Key ? Record<Key, Value> : number extends Key ? Record<Key, Value> : symbol extends Key ? Record<Key, Value> : Partial<Record<Key, Value>>;
type ReorderedArray<T extends IterableContainer> = {
    -readonly [P in keyof T]: T[number];
};

export type { ExactRecord as E, IterableContainer as I, Mapped as M, NonEmptyArray as N, ObjectKeys as O, ReadonlyTuple as R, NarrowedTo as a, ReorderedArray as b };
