import { O as OmitIndexSignature, M as Merge, P as PickIndexSignature } from './merge-qzZ2B1Xo.js';
import './simplify-BVYdScmB.js';

type ConditionalSimplifyDeep<Type, ExcludeType = never, IncludeType = unknown> = Type extends ExcludeType ? Type : Type extends IncludeType ? {
    [TypeKey in keyof Type]: ConditionalSimplifyDeep<Type[TypeKey], ExcludeType, IncludeType>;
} : Type;

type UnknownRecord = Record<string, unknown>;
/**
 * Deeply simplifies an object excluding iterables and functions. Used internally to improve the UX and accept both interfaces and type aliases as inputs.
 */
type SimplifyDeep<Type> = ConditionalSimplifyDeep<Type, Function | Iterable<unknown>, object>;
/**
 * Try to merge two record properties or return the source property value, preserving `undefined` properties values in both cases.
 */
type MergeDeepRecordProperty<Destination, Source> = undefined extends Source ? MergeDeepOrReturn<Source, Exclude<Destination, undefined>, Exclude<Source, undefined>> | undefined : MergeDeepOrReturn<Source, Destination, Source>;
/**
 * Walk through the union of the keys of the two objects and test in which object the properties are defined.
 * Rules:
 * 1. If the source does not contain the key, the value of the destination is returned.
 * 2. If the source contains the key and the destination does not contain the key, the value of the source is returned.
 * 3. If both contain the key, try to merge according to the chosen {@link MergeDeepOptions options} or return the source if unable to merge.
 */
type DoMergeDeepRecord<Destination extends UnknownRecord, Source extends UnknownRecord> = {
    [Key in keyof Destination as Key extends keyof Source ? never : Key]: Destination[Key];
} & {
    [Key in keyof Source as Key extends keyof Destination ? never : Key]: Source[Key];
} & {
    [Key in keyof Source as Key extends keyof Destination ? Key : never]: MergeDeepRecordProperty<Key extends keyof Destination ? Destination[Key] : never, Source[Key]>;
};
/**
 * Wrapper around {@link DoMergeDeepRecord} which preserves index signatures.
 */
type MergeDeepRecord<Destination extends UnknownRecord, Source extends UnknownRecord> = DoMergeDeepRecord<OmitIndexSignature<Destination>, OmitIndexSignature<Source>> & Merge<PickIndexSignature<Destination>, PickIndexSignature<Source>>;
/**
 * Try to merge two objects or two arrays/tuples recursively into a new type or return the default value.
 */
type MergeDeepOrReturn<DefaultType, Destination, Source> = SimplifyDeep<[
    undefined
] extends [Destination | Source] ? DefaultType : Destination extends UnknownRecord ? Source extends UnknownRecord ? MergeDeepRecord<Destination, Source> : DefaultType : DefaultType>;
type MergeDeep<Destination extends UnknownRecord, Source extends UnknownRecord> = SimplifyDeep<MergeDeepRecord<SimplifyDeep<Destination>, SimplifyDeep<Source>>>;

/**
 * Merges the `source` object into the `destination` object. The merge is similar to performing `{ ...destination, ... source }` (where disjoint values from each object would be copied as-is, and for any overlapping props the value from `source` would be used); But for *each prop* (`p`), if **both** `destination` and `source` have a **plain-object** as a value, the value would be taken as the result of recursively deepMerging them (`result.p === deepMerge(destination.p, source.p)`).
 *
 * @param destination - The object to merge into. In general, this object would have it's values overridden.
 * @param source - The object to merge from. In general, shared keys would be taken from this object.
 * @returns The merged object.
 * @signature
 *    R.mergeDeep(destination, source)
 * @example
 *    R.mergeDeep({ foo: 'bar', x: 1 }, { foo: 'baz', y: 2 }) // => { foo: 'baz', x: 1, y: 2 }
 * @dataFirst
 * @category Object
 */
declare function mergeDeep<Destination extends Record<string, unknown>, Source extends Record<string, unknown>>(destination: Destination, source: Source): MergeDeep<Destination, Source>;
/**
 * Merges the `source` object into the `destination` object. The merge is similar to performing `{ ...destination, ... source }` (where disjoint values from each object would be copied as-is, and for any overlapping props the value from `source` would be used); But for *each prop* (`p`), if **both** `destination` and `source` have a **plain-object** as a value, the value would be taken as the result of recursively deepMerging them (`result.p === deepMerge(destination.p, source.p)`).
 *
 * @param source - The object to merge from. In general, shared keys would be taken from this object.
 * @returns The merged object.
 * @signature
 *    R.mergeDeep(source)(destination)
 * @example
 *    R.pipe(
 *      { foo: 'bar', x: 1 },
 *      R.mergeDeep({ foo: 'baz', y: 2 }),
 *    );  // => { foo: 'baz', x: 1, y: 2 }
 * @dataLast
 * @category Object
 */
declare function mergeDeep<Destination extends Record<string, unknown>, Source extends Record<string, unknown>>(source: Source): (target: Destination) => MergeDeep<Destination, Source>;

export { mergeDeep };
